@@ -63,6 +63,118 @@ class TreeWidget(QtWidgets.QTreeWidget):
         clipboard = QtWidgets.QApplication.clipboard()
         clipboard.setText(copiedText)
 
+# text widget that allows us to push msg data directly into the text fields
+class MsgTextWidget(QtWidgets.QWidget):
+    def __init__(self, msgClass, allowedFields=[]):
+        super(MsgTextWidget, self).__init__()
+        self.grid = QtWidgets.QGridLayout()
+        self.setLayout(self.grid)
+        from msgtools.lib.unknownmsg import UnknownMsg
+        if msgClass == UnknownMsg:
+            self.showHeader = True
+        else:
+            self.showHeader = False
+        self.allowedFields = dict((fieldName,True) for fieldName in allowedFields)
+        tableHeader = []
+        if self.fieldAllowed("Time"):
+            timeUnits = Messaging.findFieldInfo(Messaging.hdr.fields, "Time").units
+            if timeUnits == "ms":
+                timeUnits = "s"
+            tableHeader.append("Time ("+timeUnits+")")
+        if self.showHeader:
+            for fieldInfo in Messaging.hdr.fields:
+                if len(fieldInfo.bitfieldInfo) == 0:
+                    if self.fieldAllowed(fieldInfo.name):
+                        tableHeader.append(fieldInfo.name)
+                else:
+                    for bitInfo in fieldInfo.bitfieldInfo:
+                        if self.fieldAllowed(bitInfo.name):
+                            tableHeader.append(bitInfo.name)
+        for fieldInfo in msgClass.fields:
+            if self.fieldAllowed(fieldInfo.name):
+                tableHeader.append(fieldInfo.name)
+            for bitInfo in fieldInfo.bitfieldInfo:
+                if self.fieldAllowed(bitInfo.name):
+                    tableHeader.append(bitInfo.name)
+        self.setHeaderLabels(tableHeader)
+    
+    def fieldAllowed(self, fieldName):
+        if len(self.allowedFields) == 0:
+            return True
+        return fieldName in self.allowedFields
+
+    def setHeaderLabels(self, headerLabels):
+        i = 0
+        for h in headerLabels:
+            self.grid.addWidget(QtWidgets.QLabel(h), 0, i)
+            self.grid.addWidget(QtWidgets.QLabel(''), 1, i)
+            i += 1
+    
+    def setDataLabels(self, dataLabels, dataAlerts):
+        i = 0
+        for t in dataLabels:
+            self.grid.itemAtPosition(1,i).widget().setText(t)
+            if dataAlerts[i]:
+                #TODO Change text color
+                pass
+            i += 1
+
+    def addData(self, msg, autoscroll=1):
+        msgStringList = []
+        columnAlerts = []
+        if self.fieldAllowed("Time"):
+            try:
+                timeVal = msg.hdr.GetTime()
+                timeInfo = Messaging.findFieldInfo(msg.hdr.fields, "Time")
+                if timeInfo.units == "ms":
+                    timeVal = timeVal / 1000.0
+                timeVal = datetime.datetime.fromtimestamp(timeVal, datetime.timezone.utc)
+            except AttributeError:
+                timeVal = datetime.datetime.now()
+            timeVal = timeVal.strftime('%H:%M:%S.%f')[:-3]
+            msgStringList.append(timeVal)
+            columnAlerts.append(0)
+        if self.showHeader:
+            for fieldInfo in Messaging.hdr.fields:
+                if len(fieldInfo.bitfieldInfo) == 0:
+                    if self.fieldAllowed(fieldInfo.name):
+                        fieldValue = str(Messaging.get(msg.hdr, fieldInfo))
+                        msgStringList.append(fieldValue)
+                else:
+                    for bitInfo in fieldInfo.bitfieldInfo:
+                        if self.fieldAllowed(bitInfo.name):
+                            fieldValue = str(Messaging.get(msg.hdr, bitInfo))
+                            msgStringList.append(fieldValue)
+        for fieldInfo in type(msg).fields:
+            if(fieldInfo.count == 1):
+                if self.fieldAllowed(fieldInfo.name):
+                    fieldValue = str(Messaging.get(msg, fieldInfo))
+                    msgStringList.append(fieldValue)
+                    columnAlerts.append(Messaging.getAlert(msg, fieldInfo))
+                for bitInfo in fieldInfo.bitfieldInfo:
+                    if self.fieldAllowed(bitInfo.name):
+                        fieldValue = str(Messaging.get(msg, bitInfo))
+                        msgStringList.append(fieldValue)
+                        columnAlerts.append(Messaging.getAlert(msg, bitInfo))
+            else:
+                if self.fieldAllowed(fieldInfo.name):
+                    columnText = ""
+                    alert = 0
+                    for i in range(0,fieldInfo.count):
+                        fieldValue = Messaging.get(msg, fieldInfo, i)
+                        # if the value is what is given when we go off the end of an array, break.
+                        if fieldInfo.type == "int" and fieldValue == "UNALLOCATED":
+                            break
+                        columnText += str(fieldValue)
+                        if Messaging.getAlert(msg, fieldInfo, i):
+                            alert = 1
+                        if(i<fieldInfo.count-1):
+                            columnText += ", "
+                    msgStringList.append(columnText)
+                    columnAlerts.append(alert)
+
+        self.setDataLabels(msgStringList, columnAlerts)
+
 # tree widget that allows us to push msg data directly into the tree
 class MsgTreeWidget(TreeWidget):
     MAX_ROWS = 1000
@@ -276,6 +388,7 @@ class LineEditWithHistory(QtWidgets.QLineEdit):
 class MsgCommandWidget(QtWidgets.QWidget):
     commandEntered = QtCore.pyqtSignal(str)
     messageEntered = QtCore.pyqtSignal(object)
+    autocompleted   = QtCore.pyqtSignal(str)
     def __init__(self, parent=None):
         super(MsgCommandWidget, self).__init__()
         self.textBox = QtWidgets.QPlainTextEdit()
@@ -294,6 +407,7 @@ class MsgCommandWidget(QtWidgets.QWidget):
         autocomplete, help = msgcsv.csvHelp(lineOfText)
         if autocomplete:
             self.lineEdit.setText(autocomplete)
+            self.autocompleted.emit(autocomplete)
         if help:
             self.addText('\n'+help+'\n>\n')
 
