@@ -107,7 +107,6 @@ class Messaging:
     def DetermineLoadDir(loaddir, searchdir):
         if loaddir:
             loadDir = loaddir
-            pass
         else:
             if not searchdir or not os.path.isdir(searchdir):
                 searchdir = os.getcwd()
@@ -116,11 +115,16 @@ class Messaging:
                 lastsearchdir = searchdir
                 searchdir = os.path.abspath(os.path.join(searchdir, os.pardir))
                 # checking if joining with pardir returns the same dir is the easiest way to
-                # determine if we're at root of filesystem
+                # determine if we're at root of filesystem, and our upward search has to stop.
                 if lastsearchdir == searchdir:
-                    # if we're at root of filesystem, just give up!
-                    loadDir = None
-                    raise RuntimeError('''
+                    # check for global path to generated code
+                    if os.path.isdir("/opt/msgtools/obj/CodeGenerator/Python/"):
+                        loadDir = "/opt/msgtools/obj/CodeGenerator/Python/"
+                        break
+                    else:
+                        # if we're at root of filesystem, just give up!
+                        loadDir = None
+                        raise RuntimeError('''
 ERROR! Auto-generated python code not found!
 cd to a directory downstream from a parent of obj/CodeGenerator/Python
 or specify that directory with --msgdir=PATH''')
@@ -401,6 +405,39 @@ or specify that directory with --msgdir=PATH''')
                 if name == bfi.name:
                     return bfi
         return None
+    
+    # Create a fake field.  This is useful if we want to compute data based on an
+    # existing message's fields that we want to plot or log.
+    @staticmethod
+    def createFakeField(msg_class, name, units="", minVal="", maxVal="", description="Fake field!", count=1):
+        def getFn(self, index=0):
+            if count == 1:
+                return self.fake_fields[name]
+            else:
+                return self.fake_fields[name][index]
+        getFn.units = units
+        getFn.default = ''
+        getFn.minVal = minVal
+        getFn.maxVal = maxVal
+        getFn.offset = 0
+        getFn.size = 0
+        getFn.count = count
+        def setFn(self, value, index=0):
+            if count == 1:
+                self.fake_fields[name] = value
+            else:
+                if not name in self.fake_fields:
+                    self.fake_fields[name] = []
+                self.fake_fields[name][index] = value
+        setFn.units = units
+        setFn.default = ''
+        setFn.minVal = minVal
+        setFn.maxVal = maxVal
+        setFn.offset = 0
+        setFn.size = 0
+        setFn.count = count
+        new_fi = FieldInfo(name, type="", units=units, minVal=minVal, maxVal=maxVal, description=description, get=getFn, set=setFn, count=count, bitfieldInfo=[], enum=[])
+        msg_class.fields.append(new_fi)
 
     # This is composed of all the header fields that are not length, time, and any ID fields.
     # In some systems where one PC talks to one device, there may not be *any* fields that
