@@ -15,7 +15,7 @@ class YamlLoader(yaml.Loader):
         yaml.Loader.__init__(self, stream)
 
     def include(self, node):
-        filename = os.path.join(self._dirname, node.value)
+        filename = os.path.abspath(os.path.join(self._dirname, node.value))
         try:
             inFile = io.open(filename, 'r')
             return yaml.load(inFile, YamlLoader)
@@ -44,6 +44,16 @@ def fieldSize(field):
     fieldSizes = {"uint64":8, "uint32":4, "uint16": 2, "uint8": 1, "int64":8, "int32":4, "int16": 2, "int8": 1, "float64":8, "float32":4}
     return fieldSizes[field["Type"]]
 
+def fieldLocation(field):
+    return field['Location']
+
+# offset between elements of an array of this field.
+# same as field size for most fields, except when this is an element of an array of structs
+def fieldArrayElementOffset(field):
+    if 'StructSize' in field:
+        return int(field['StructSize'])
+    return fieldSize(field)
+
 def msgSize(msg):
     offset = 0
     if "Fields" in msg:
@@ -253,33 +263,33 @@ def msgID(msg, enums, undefinedMsgId):
     ret = undefinedMsgId
     # check if there was an externally specified list of ID fields
     if msg["ids"]:
-        try:
-            ret = 0
-            # iterate over list of ids
-            shiftValue = 0
-            for id in msg["ids"]:
+        ret = 0
+        # iterate over list of ids
+        shiftValue = 0
+        for id in msg["ids"]:
+            try:
                 value = msg[id["Name"]]
-                #print("got value " + str(value))
-                try:
-                    value = int(value)
-                except ValueError:
-                    for enum in enums:
-                        enumType = enum["Name"]
-                        for option in enum["Options"]:
-                            if value == enumType + "." + option["Name"] or (enum["Name"] == id["Name"]+"s" and value == option["Name"]):
-                                enumName = value
-                                value = int(option["Value"])
-                                #print("found value " + str(value) + " for " + enumType + "." + str(enumName))
-                                break
-                try:
-                    value = int(value)
-                except ValueError:
-                    raise MessageException("ERROR! Can't find value for " + str(value))
-                shiftValue = id["Bits"]
-                #print("ID " + id["Name"] + " is " + str(value) + ", " + str(id["Bits"]) + " bits")
-                ret = (ret << shiftValue) + value
-        except KeyError:
-            pass
+            except KeyError:
+                value = 0
+            #print("got value " + str(value))
+            try:
+                value = int(value)
+            except ValueError:
+                for enum in enums:
+                    enumType = enum["Name"]
+                    for option in enum["Options"]:
+                        if value == enumType + "." + option["Name"] or (enum["Name"] == id["Name"]+"s" and value == option["Name"]):
+                            enumName = value
+                            value = int(option["Value"])
+                            #print("found value " + str(value) + " for " + enumType + "." + str(enumName))
+                            break
+            try:
+                value = int(value)
+            except ValueError:
+                raise MessageException("ERROR! Can't find value for " + str(value))
+            shiftValue = id["Bits"]
+            #print("ID " + id["Name"] + " is " + str(value) + ", " + str(id["Bits"]) + " bits")
+            ret = (ret << shiftValue) + value
     if "ID" in msg:
         ret = msg["ID"]
     #print("message " + msg["Name"] + " has ID " + hex(ret))
@@ -292,7 +302,7 @@ def msgDescriptor(msg, inputFilename):
     # add the filename as a namespace, unless it's already there
     if not basename == name and not basename+'_' in name:
         name = basename + '.' + name
-        print("adding basename " + basename + " to " + name)
+        #print("adding basename " + basename + " to " + name)
     if name.startswith(subdir+"_"):
         return name.replace("_", ".")
     if subdir and not name.startswith(subdir):
@@ -371,39 +381,77 @@ def Enums(inputData):
     return enumList
 
 def Structs(inputData):
-    structList = []
+    structList = {}
     if "Structs" in inputData:
         structList = inputData["Structs"]
     if "includes" in inputData:
         for data in inputData["includes"]:
-            structList = structList + Structs(data)
+            structList.update(Structs(data))
     return structList
 
-# this replaces fields that are references to structs with the fields from the referenced struct
+# this replaces fields that are references to structs with the fields from the referenced struct,
+# and also sets a few items of field meta-data (Location, StructSize, Count)
 def PatchStructs(inputData):
     # loop twice, so that references to structs inside structs are also replaced
     for i in range(2):
         structs = Structs(inputData)
-        if not structs:
-            return
+        #if not structs:
+        #    return
         
         # need to make a new list, because we'll be inserting elements as we iterate
         if 'Messages' in inputData:
             for msg in inputData["Messages"]:
+                resolve_sizes_and_locations = True
                 if 'Fields' in msg:
+                    location = 0
                     outfields = []
                     for field in msg['Fields']:
                         if field["Type"] in structs:
                             s = structs[field["Type"]]
+                            # Compute the size of the structure and store it in each field in the structure.
+                            # This is useful for finding subsequent elements of fields of arrays of structs,
+                            # because we'll treat them as parallel arrays in the generated code, not as
+                            # hierarchical structs.
+                            structSize = 0
+                            for subfield in s['Fields']:
+                                if resolve_sizes_and_locations:
+                                    try:
+                                        structSize += fieldSize(subfield)
+                                    except KeyError:
+                                        # if we have a KeyError when trying to get fieldSize,
+                                        # it's because we have a nested Struct that hasn't been
+                                        # replaced by it's members yet.  Don't worry about that now,
+                                        # because we'll iterate and replace it in the next loop.
+                                        resolve_sizes_and_locations = False
+                                        structSize = None
+                                        location = None
+                            # track field offsets within struct
+                            subfieldLocation = 0
                             for subfield in s['Fields']:
                                 subfieldcopy = copy.deepcopy(subfield)
                                 subfieldcopy['Name'] = field['Name'] + "_" + subfield['Name']
+                                if resolve_sizes_and_locations:
+                                    subfieldcopy['StructSize'] = structSize
+                                    subfieldcopy['Location'] = location + subfieldLocation
+                                # inherit count from parent field, so that if it's an array,
+                                # we'll be an array too.
+                                subfieldcopy['Count'] = fieldCount(field)
+                                #TODO How to handle array of bitfields?!?
                                 if "Bitfields" in subfieldcopy:
                                     for bits in subfieldcopy["Bitfields"]:
                                         bits['Name'] = field['Name'] + "_" + bits['Name']
                                 outfields.append(subfieldcopy)
+                                if resolve_sizes_and_locations:
+                                    subfieldLocation += fieldSize(subfield)
+                            if resolve_sizes_and_locations:
+                                # add size of struct times struct count to running total of field location
+                                location += subfieldLocation * fieldCount(field)
                         else:
                             outfields.append(field)
+                            if resolve_sizes_and_locations:
+                                if not 'Location' in field:
+                                    field['Location'] = location
+                                location += fieldSize(field) * fieldCount(field)
                     msg['Fields'] = outfields
 
 # sanitize the option name, to have valid identifier characters
