@@ -17,7 +17,6 @@ from PyQt5 import QtCore, QtGui, QtWidgets, QtNetwork
 
 from msgtools.server.TcpServer import *
 from msgtools.server.WebSocketServer import *
-import msgtools.lib.msgjson as msgjson
 
 DESCRIPTION='''
     MsgServer acts as a central routing hub for one or more message clients.
@@ -180,6 +179,14 @@ class MessageServer(QtWidgets.QMainWindow):
         parser.add_argument('--port', type=int, 
             help='The TCP port to use.  Websockets are always TCP port + 1.')
 
+        # if we had plugins before, but no cmdline args now, add simulated
+        # cmdline args for our old plugins
+        if len(sys.argv) == 1:
+            last_plugins = self.settings.value("pluginsLoaded", "", str).split("|")
+            for plugin in last_plugins:
+                if plugin:
+                    sys.argv.append("--"+plugin)
+
         args = parser.parse_args()
         
         try:
@@ -205,14 +212,14 @@ class MessageServer(QtWidgets.QMainWindow):
         if args.port is not None:
             tcpport = args.port
             wsport = tcpport+1
-
+        
         for entry_point in pkg_resources.iter_entry_points("msgtools.server.plugin"):
             # check for argparse data for the plugin
             plugin_option = getattr(args, entry_point.name)
             plugin_last_option = getattr(args, 'last'+entry_point.name)
             if plugin_last_option is not False or plugin_option is not None:
                 param = plugin_option if plugin_option is not None else None
-                self.load_plugin(entry_point, param)
+                self.load_plugin(entry_point.name, entry_point, param)
 
         # load plugin via path to .py file
         if args.plugin is not None:
@@ -283,13 +290,24 @@ class MessageServer(QtWidgets.QMainWindow):
         self.setWindowTitle("MessageServer 0.1")
         self.statusBar()
 
-    def startLog(self, logFileName):
+    def startLog(self, log_name):
         if self.logFile:
             self.logFile.close()
             self.logFile = None
-        self.logFile = QtCore.QFile(logFileName)
+        if log_name.endswith("json"):
+            self.logFileType = "json"
+        elif log_name.endswith("csv"):
+            self.logFileType = "csv"
+            # hash table of booleans for if each type of message has had it's header
+            # put into this log file already.
+            self.loggedMsgHeaderRow = {}
+        elif log_name.endswith('bin') or log_name.endswith('log'):
+            self.logFileType = "bin"
+        else:
+            print("ERROR!  Invalid log type " + log_name)
+        self.logFile = QtCore.QFile(log_name)
         self.logFile.open(QtCore.QIODevice.Append)
-        fileInfo = QtCore.QFileInfo(logFileName)
+        fileInfo = QtCore.QFileInfo(log_name)
         self.settings.setValue("logging/filename", fileInfo.dir().absolutePath())
         self.logButton.setText("Stop " + fileInfo.fileName())
         self.queryLog()
@@ -307,8 +325,12 @@ class MessageServer(QtWidgets.QMainWindow):
             if self.logFile != None:
                 logStatusMsg.SetLogOpen(1)
                 logStatusMsg.SetLogFileName(self.logFile.fileName())
-                if self.logFileType == "JSON":
+                if self.logFileType == "json":
                     logStatusMsg.SetLogFileType("JSON")
+                elif self.logFileType == "csv":
+                    logStatusMsg.SetLogFileType("CSV")
+                elif self.logFileType == "bin":
+                    logStatusMsg.SetLogFileType("BIN")
             for client in self.clients.values():
                 client.sendMsg(logStatusMsg.hdr)
 
@@ -318,7 +340,7 @@ class MessageServer(QtWidgets.QMainWindow):
         else:
             currentDateTime = QtCore.QDateTime.currentDateTime()
             defaultFilename = currentDateTime.toString("yyyyMMdd-hhmmss") + ".log"
-            logFileName, _ = QtWidgets.QFileDialog.getSaveFileName(self, "Save File", self.settings.value("logging/filename", ".")+"/"+defaultFilename, "Log Files (*.log)")
+            logFileName, _ = QtWidgets.QFileDialog.getSaveFileName(self, "Save File", self.settings.value("logging/filename", ".")+"/"+defaultFilename, "Log Files (*.log);;JSON Files (*.json);;CSV Files (*.csv)")
             # if they hit cancel, don't do anything
             if not logFileName:
                 return
@@ -332,12 +354,22 @@ class MessageServer(QtWidgets.QMainWindow):
     def pluginSelected(self, plugin_name):
         for entry_point in pkg_resources.iter_entry_points("msgtools.server.plugin"):
             if entry_point.name == plugin_name:
-                self.load_plugin(entry_point, None)
+                self.load_plugin(plugin_name, entry_point, None)
                 return
 
-    def load_plugin(self, entry_point, param):
-        pluginCreatorFn = entry_point.load()
-        pluginPort = pluginCreatorFn()
+    def load_plugin(self, plugin_name, entry_point, param):
+        try:
+            # load the module, create the plugin
+            pluginCreatorFn = entry_point.load()
+            pluginPort = pluginCreatorFn()
+            pluginPort.plugin_name = plugin_name
+        except:
+            a,b,c = sys.exc_info()
+            exc = ''.join(traceback.format_exception(a,b,c))
+            self.onStatusUpdate("Exception loading plugin:\n%s" % (exc))
+            return
+        
+        # connect plugin to our status update function
         pluginPort.statusUpdate.connect(self.onStatusUpdate)
         
         # try to allow plugin to emit new connections
@@ -366,9 +398,14 @@ class MessageServer(QtWidgets.QMainWindow):
                 break
             self.grid.addWidget(widget, clientRow, i)
             i+=1
-
+    
     def onConnectionDied(self, connection):
         self.onStatusUpdate("removing connection[" + connection.name+"]")
+        # remove the plugin, if it is one
+        try:
+            self.pluginPorts.remove(connection)
+        except ValueError:
+            pass
         i = 0
         while(1):
             widget = connection.widget(i)
@@ -383,14 +420,25 @@ class MessageServer(QtWidgets.QMainWindow):
         else:
             self.onStatusUpdate("cnx not in list!")
 
-    def logMessage(self, hdr):
+    def logMsg(self, hdr):
         #write to log, if log is open
         if self.logFile != None:
-            if self.logFileType and self.logFileType == "JSON":
-                msgObj = Messaging.MsgFactory(hdr)
-                self.logFile.write(msgjson.toJson(msgObj).encode('utf-8'))
-            else:
-                self.logFile.write(hdr.rawBuffer().raw)
+            log = ''
+            if self.logFileType == "csv":
+                msg = Messaging.MsgFactory(hdr)
+                if not msg.MsgName() in self.loggedMsgHeaderRow:
+                    self.loggedMsgHeaderRow[msg.MsgName()] = True
+                    log = msg.csvHeader(timeColumn=True)+'\n'
+                log += msg.toCsv(timeColumn=True)+'\n'
+                log = log.encode('utf-8')
+            elif self.logFileType == "json":
+                msg = Messaging.MsgFactory(hdr)
+                log = msg.toJson(includeHeader=True)+'\n'
+                log = log.encode('utf-8')
+            elif self.logFileType == "bin":
+                log = hdr.rawBuffer().raw
+            self.logFile.write(log)
+            self.logFile.flush()
 
     def onMessageReceived(self, hdr):
         c = self.sender()
@@ -431,7 +479,7 @@ class MessageServer(QtWidgets.QMainWindow):
         elif hasattr(self.networkMsgs, 'Note') and hdr.GetMessageID() == self.networkMsgs.Note.ID:
             # The Note message allows a user to annotate a log.  We want to drop it into the log, but not
             # forward to all other clients.
-            self.logMessage(hdr)
+            self.logMsg(hdr)
 
         elif hasattr(self.networkMsgs, 'PrivateSubscriptionList') and  hdr.GetMessageID() == self.networkMsgs.PrivateSubscriptionList.ID:
             subListMsg = self.networkMsgs.PrivateSubscriptionList(hdr.rawBuffer())
@@ -448,7 +496,7 @@ class MessageServer(QtWidgets.QMainWindow):
             self.onStatusUpdate("adding Private subscription for "+c.name+": " + ', '.join(hex(x) for x in privateSubs))
         else:
             # Log the message
-            self.logMessage(hdr)
+            self.logMsg(hdr)
 
             # Route to all clients
             for client in self.clients.values():
@@ -470,8 +518,14 @@ class MessageServer(QtWidgets.QMainWindow):
     def closeEvent(self, event):
         self.settings.setValue("geometry", self.saveGeometry())
         self.settings.setValue("windowState", self.saveState())
+        pluginNames = []
         for pluginPort in self.pluginPorts:
+            try:
+                pluginNames.append(pluginPort.plugin_name)
+            except AttributeError:
+                pass
             pluginPort.stop()
+        self.settings.setValue("pluginsLoaded", "|".join(pluginNames))
         super(MessageServer, self).closeEvent(event)
 
     def readSettings(self):
